#!/pro/bin/perl

use strict;
use warnings;
use integer;
use DB_File;
use Getopt::Long;

my ($list, %db, $id, $age) = (4);
for (qw(name points country age)) {
    tie %{$db{$_}}, "DB_File", "db/$_";
    }

GetOptions (
    "l" => sub { $list = 1 },	# plain List
    "b" => sub { $list = 2 },	# plain Barchart
    "h" => sub { $list = 3 },	# barchart per Hat/Age
    "a" => sub { $list = 4 },	# barchart per Age/Hat
    ) or die;

while (print STDERR "ID : ") {
    chomp ($id = uc scalar <STDIN>);
    $id =~ s/^-/CA/;
    $id =~ s/^\+/BR/;
    $id =~ m/^\w+/ or last;
    ($id, $age) = split m/\W+/, $id, 2;
    unless (exists $db{points}{$id}) {
	print STDERR "No points for $id\nStill add? [y/N] > N\b";
	scalar <STDIN> =~ m/^[JjYy+]/ or next;
	$db{points}{$id} = 0;
	}
    unless ($age) {
	print "age: ";
	chomp ($age = <STDIN>);
	}
    if (exists $db{age}{$id}) {
	print STDERR "$id\'s age was already known ($db{age}{$id})\n";
	$age == $db{age}{$id} or
	    warn "Changing age for $id from $db{age}{$id} to $age\n";
	}
    $age and $db{age}{$id} = $age + 0;
    }

my %age_db = %{$db{age}};
untie %{$db{age}};
delete $age_db{CA76779};

my %hats = qw( Bold 250 Pro 500 Graduate 1000 Wizard 2500 Royalty 7500 Pharaoh 20000 Olympian 999999);

if ($list == 1) {
    foreach $id (sort { $age_db{$a} <=> $age_db{$b} } keys %age_db) {
	printf "%2d %-12s %-20.20s %s\n", $age_db{$id}, $id, $db{name}{$id}, $db{country}{$id};
	}
    exit;
    }

if ($list == 2) {
    my %age;
    $age{$_}++ for values %age_db;
    my %hist;
    $hist{$_ / 10} += $age{$_} for keys %age;
    printf "%d0 - %d9 %3d\n", $_, $_, $hist{$_} for sort { $a <=> $b } keys %hist;
    exit;
    }

if ($list == 3) {
    my $low = 0;
    foreach my $hat (sort { $hats{$a} <=> $hats{$b} } keys %hats) {
	my $upper = $hats{$hat} == 999999 ? "" : $hats{$hat} - 1;
	print "$hat $low .. $upper\n";
	my %age;
	foreach my $id (keys %age_db) {
	    $db{points}{$id} > $low && $db{points}{$id} < $hats{$hat} or next;
	    #printf "    %2d %-12s %-20.20s %s\n", $age_db{$id}, $id, $db{name}{$id}, $db{country}{$id};
	    $age{$age_db{$id}}++
	    }
	$low = $hats{$hat};
	my %hist;
	$hist{$_ / 10} += $age{$_} for keys %age;
	printf "  %d0 - %d9 %3d\n", $_, $_, $hist{$_} for sort { $a <=> $b } keys %hist;
	}
    exit;
    }

my (@cnt, $age_range);

format XLS_TOP =
+---------+-----+-----+-----+-----+-----+-----+-----+-----+
| Age     | Bld | Pro | Gra | Wiz | Roy | Pha | Oly |     |
+---------+-----+-----+-----+-----+-----+-----+-----+-----+
.
format XLS =
| @>>>>>> | @>> | @>> | @>> | @>> | @>> | @>> | @>> | @>> |
$age_range, @cnt
+---------+-----+-----+-----+-----+-----+-----+-----+-----+
.

if ($list == 4) {
    my %hist;
    my $gran = 5;
    foreach my $id (keys %age_db) {
	my $low = 0;
	foreach my $hat (sort { $hats{$a} <=> $hats{$b} } keys %hats) {
	    $db{points}{$id} > $low && $db{points}{$id} < $hats{$hat} or next;
	    $hist{$age_db{$id} / $gran}{$hat}++;
	    $hist{$age_db{$id} / $gran}{sum}++;
	    last;
	    }
	}
    local $~ = "XLS";
    for (sort { $a <=> $b } keys %hist) {
	$age_range = sprintf "%02d - %02d", $gran * $_, $gran * $_ + $gran - 1;
	my $low = 0;
	@cnt = ();
	foreach my $hat (sort { $hats{$a} <=> $hats{$b} } keys %hats) {
	    push @cnt, exists $hist{$_}{$hat} ? $hist{$_}{$hat} : "-";
	    $low = $hats{$hat};
	    }
	push @cnt, $hist{$_}{sum};
	{ no warnings;
	  foreach my $i (0..$#cnt) { $hats{sum}[$i] += $cnt[$i] }}
	write;
	}
    $age_range = ""; @cnt = @{$hats{sum}}; delete $hats{sum};
    write;
    }
